﻿// Spyrograph.as - Jim Bumgardner

SW = Stage.width;
SH = Stage.height;
CX = Stage.width/2;
CY = (Stage.height-100)/2;
kNumberTrails = 200;

ringRad  = SW/2;
discRadR = .81;
penRadR = .7;
speed = 0.6;
lineThick = 4;
showGuide = true;
lastX = CX;
lastY = CY;
angle = 0;
isTiger = false;

guide_mc = _root.createEmptyMovieClip("guide_mc", 50);

// This is used for drawing the guide
MovieClip.prototype.drawCircle = function(x,y,radius)
{
	var r = radius;
	var degToRad = Math.PI/180;
	var n= 8;
	
	var theta = 45*degToRad;
	var cr = radius/Math.cos(theta/2);
	var angle = bA*degToRad;
	var cangle = angle-theta/2;
	
	this.moveTo(x+r, y);
	for (var i=0;i < n;i++) 
	{
		angle += theta;
		cangle += theta;
		var endX = r*Math.cos (angle);
		var endY = r*Math.sin (angle);
		var cX = cr*Math.cos (cangle);
		var cY = cr*Math.sin (cangle);
		this.curveTo(x+cX,y+cY, x+endX,y+endY);
	}
}


guide_mc.onEnterFrame = function()
{
  var discRad = ringRad*discRadR;
  var penRad = discRad*penRadR;
  this.clear();

  // Basic spyrograph math...

  // angle of disc
  if (!_root.isAdjusting)
	  angle += discRadR*speed;
  
  // center of disc
  csx = CX+Math.cos(angle)*(ringRad-discRad);
  csy = CX+Math.sin(angle)*(ringRad-discRad);

  // angle of pen from center of disc
  if (!_root.isAdjusting)
	  angPen = angle-angle*ringRad/discRad;
  
  // position of pen
  lastX = csx+Math.cos(angPen)*penRad;
  lastY = csy+Math.sin(angPen)*penRad;

  // compute point we're curving thru (cx,cy) for curveTo command
  var angle2 = (angle-(speed*discRadR)/2);
  var csx2 = CX+Math.cos(angle2)*(ringRad-discRad);
  var csy2 = CX+Math.sin(angle2)*(ringRad-discRad);
  var angPen2 = angle2-angle2*ringRad/discRad;
  var cRad = penRad/Math.cos(angPen-angPen2);
  var lcx = csx2+Math.cos(angPen2)*cRad;
  var lcy = csy2+Math.sin(angPen2)*cRad;
 
  if (showGuide || _root.isAdjusting) 
  {  
     var alphaScale = _root.isAdjusting? 1 : .2;
     this.lineStyle(4,0x888888,100*alphaScale);
     this.drawCircle(csx,csy,discRad);
     this.lineStyle(2,0x888888,100*alphaScale);
     this.moveTo(csx+Math.cos(angPen)*discRad, csy+Math.sin(angPen)*discRad);
     this.lineTo(csx-Math.cos(angPen)*discRad, csy-Math.sin(angPen)*discRad);
     this.lineStyle(4,0x888888,100*alphaScale);
     this.drawCircle(lastX,lastY,1+lineThick);
  }

  if (!isAdjusting)
  {
	  for (var i = 0; i < kNumberTrails; i++)
	  {
		dotArray[i].x = dotArray[i+1].x;
		dotArray[i].y = dotArray[i+1].y;
		dotArray[i].cx = dotArray[i+1].cx;
		dotArray[i].cy = dotArray[i+1].cy;
	  }
	  dotArray[kNumberTrails].x = lastX;
	  dotArray[kNumberTrails].y = lastY;
	  dotArray[kNumberTrails].cx = lcx;
	  dotArray[kNumberTrails].cy = lcy;
  }

  var alphaScale = _root.isAdjusting? .1 : 1;
  this.moveTo(dotArray[0].x, dotArray[0].y);
	
  for (var i = 0; i <= kNumberTrails; i++) 
  {
    this.lineStyle(lineThick,dotArray[i].clr, dotArray[i].alpha*alphaScale);
    this.curveTo(dotArray[i].cx,dotArray[i].cy, dotArray[i].x, dotArray[i].y);
  }
}


getRainbowColor = function(r) // r is 0 - 1  - returns a bright color
{
	var a = r*Math.PI*2;
	var r = 128+Math.cos(a)*127;
	var b = 128+Math.cos(a+2*Math.PI/3)*127;
	var g = 128+Math.cos(a+4*Math.PI/3)*127;
	return (r << 16) | (g << 8) | b;
}

getTigerColor = function(r) // r is 0 - 1  - returns a bright color
{
	var a = r*Math.PI*2*11;
	var r = 128+Math.cos(a)*127;
	var b = 64+Math.sin(a)*64;
	var g = 64+Math.cos(a)*64;
	return (r << 16) | (g << 8) | b;
}


// get initial pen position...
var discRad = ringRad*discRadR;
var penRad = discRad*penRadR;
var csx = CX+Math.cos(0)*(ringRad-discRad);
var csy = CX+Math.sin(0)*(ringRad-discRad);
lastX = csx+Math.cos(0)*penRad; // pen position
lastY = csy+Math.sin(0)*penRad;

dotArray = [];
for (i = 0; i <= kNumberTrails; ++i) {
	var elem = [];
	elem.x = elem.cx = lastX;
	elem.y = elem.cy = lastY;
	elem.alpha = i < 20? i*100/20 : 100;
	dotArray[i] = elem;
}

function adjustColor()
{
	for (var i = 0; i <= kNumberTrails; ++i) 
	{
		if (isTiger)
			dotArray[i].clr = getTigerColor(i/kNumberTrails);
		else
			dotArray[i].clr = getRainbowColor(i/kNumberTrails);
	}
}

adjustColor();

resetDots = function()
{
	for (var i = 0; i < kNumberTrails; ++i) 
	{
		dotArray[i].x = dotArray[kNumberTrails].x;
		dotArray[i].y = dotArray[kNumberTrails].y;
		dotArray[i].cx = dotArray[kNumberTrails].cx;
		dotArray[i].cy = dotArray[kNumberTrails].cy;
	}
}

// GUI STUFF - not strictly necessary for spyrograph stuff...
#include "jbumUI.as"

slider1_mc.setupSlider('disc size',0,1, discRadR,function(t) { discRadR = t; } );
slider2_mc.setupSlider('pen hole',0,1, penRadR,function(t) { penRadR = t; } );
slider3_mc.setupSlider('thickness',0,20, lineThick,function(t) { lineThick = t; } );
button1_mc.setupToggleButton("guide", showGuide, function(t) { showGuide = t; }, true);
button2_mc.setupButton("clear", function() { resetDots(); }, true);
button3_mc.setupToggleButton("tiger", isTiger, function(t) { isTiger = t; adjustColor(); }, true);
